#include "Engine.h"

extern "C" __declspec(dllexport)
void textureffgLoader(char * name, int id)
{
    int bits, size;
    GLenum type;
    unsigned char header[7];
    int height, width;
    FILE * file;
    
    file = fopen(name, "rb");
    if (file == NULL)
    {
        printf("could not find image\n");
        return;
    }    
	fread (header, sizeof(char), 8, file);
	
	if (header[0] != 73)
	{
        printf("Not a valid ffg image\n");
        return;
    }
    
    if (header[1] != 1)
	{
        printf("ffg type not supported\n");
        return;
    }
	width = header[3] + header[4] * 256;
    height = header[5] + header[6] * 256;
    
    if ((width != 2 && width != 4 && width != 8 && width != 16 && width != 32  
    && width != 64 && width != 128 && width != 256 && width != 512 && width != 1024
    && width != 2048 && width != 4096 && width != 8192 && width != 16384 && width != 32768) 
    && (height != 2 && height != 4 && height != 8 && height != 16 && height != 32  
    && height != 64 && height != 128 && height != 256 && height != 512 && height != 1024
    && height != 2048 && height != 4096 && height != 8192 && height != 16384 && height != 32768))
    {
        printf("Image dimensions are not supported");
        return;
    }
    
	if (header[2] == 32)
	{
	    size = 4;
        type = GL_RGBA;
    }    
    else if (header[2] == 24)
    {
        size = 3;
        type = GL_RGB;
    }
    else
        return; 
    unsigned char data[height * width * size];
    fread(data, sizeof(data), 1, file);
     
	fclose(file);
	glBindTexture (GL_TEXTURE_2D, id);
    glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexImage2D(GL_TEXTURE_2D, 0, type, width, height, 0, type, GL_UNSIGNED_BYTE,data);
}

extern "C" __declspec(dllexport)
void texturebmpLoader(char * name, int id)
{
    int j = 0;
    FILE *file;
    BITMAPFILEHEADER fileheader; 
    BITMAPINFOHEADER infoheader;
    RGBTRIPLE rgb;
    unsigned char header[4] = {0, 0, 0, 0};
    unsigned char * texture = NULL;
    
    if((file = fopen(name, "rb"))==NULL)
    {
        printf("could not find one or more of the images\n");
        return;
    }
    
    fread(&fileheader, sizeof(fileheader), 1, file);
    fseek(file, sizeof(fileheader), SEEK_SET);
    fread(&infoheader, sizeof(infoheader), 1, file);

    texture = (byte *) malloc(infoheader.biWidth * infoheader.biHeight * 4);
 
    for (int i = 0; i < infoheader.biWidth * infoheader.biHeight; i++)
    {            
        fread(&rgb, sizeof(rgb), 1, file); 
        texture[j+0] = rgb.rgbtRed; 
        texture[j+1] = rgb.rgbtGreen; 
        texture[j+2] = rgb.rgbtBlue; 
        texture[j+3] = 255;
        j += 4;
    }
    fclose(file);
    
    glBindTexture (GL_TEXTURE_2D, id);
    glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, infoheader.biWidth, infoheader.biHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE,texture);
}

extern "C" __declspec(dllexport)
void texturetgaLoader(char * name, int id)
{
    FILE * file;
    GLenum type;
    unsigned char is_the_file_a_good_type[4];
    unsigned char picture_data[7];
    unsigned char * data = NULL;
    int width, height;
    int bits, size;
    
    if (!(file = fopen(name, "rb")))
    {
        printf("could not find image\n");
        return;
    }
    
    fread (&is_the_file_a_good_type, sizeof(char), 3, file);
    if (is_the_file_a_good_type[2] != 2)
    {
        fclose(file);
        printf("image type is not supported\n");
        return;
    }
    
    fseek (file, 12, SEEK_SET);
    fread (&picture_data, sizeof (char), 6, file);
    width = picture_data[0] + picture_data[1] * 256;
    height = picture_data[2] + picture_data[3] * 256;
    bits =	picture_data[4]; 
    size = height * width;
    
    if (bits == 32)
    {
            unsigned char * pdata;
            unsigned char temp;
            int amount_of_data_read;

            pdata = (unsigned char*) malloc(size * 4);

            if (pdata == NULL)
            {
                printf("no more memory\n");
                return;
            }    

            amount_of_data_read = fread (pdata, sizeof (unsigned char), size * 4, file);
            if (amount_of_data_read != size * 4)
            {
                free(pdata);
                printf("problems with file format\n");
                return;
            }

            for (int c = 0; c < size * 4; c = c + 4 )
            {
                temp = pdata[c];
                pdata[c] = pdata[c + 2];
                pdata[c + 2] = temp;
            }

            type = GL_RGBA;
            data = pdata;
    }
    else if (bits == 24)
    {
        unsigned char * pdata2;
        unsigned char temp;
        int amount_of_data_read;

        pdata2 = (unsigned char*) malloc(size * 3); 

        if (pdata2 == NULL)
        {
            printf("no more memory\n");
            return;
        }    

        amount_of_data_read = fread (pdata2, sizeof (unsigned char), size * 3, file);

        if (amount_of_data_read != size * 3)
        {
            free (pdata2);
            printf("problems with file format\n");
            return;
        }

        for (int c = 0; c < size * 3; c = c + 3)
        {
            temp = pdata2[c];
            pdata2[c] = pdata2[c + 2];
            pdata2[c + 2] = temp;
        }

        type = GL_RGB;
        data = pdata2;
    }
    
    if (data == NULL)
    {
        fclose(file);
        printf("problems with file format\n");
        return;
    } 
    
    fclose(file);
    
    glBindTexture (GL_TEXTURE_2D, id);
    glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexImage2D (GL_TEXTURE_2D, 0, type, width, height, 0, type, GL_UNSIGNED_BYTE, data);
}
